#!/bin/sh -f

#------------------------------------------------------------------------------
# slides.sh, an HTML presentation generator 
Version=1.5
# Copyright (C) 2005 Francois-Xavier Le Bail
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details (file : COPYING).

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#------------------------------------------------------------------------------
# Usage : slides.sh [options] [source_file]
# By default : the name of the source file is : source.txt
#             create slide001.html, slide002.html, etc. in the current dir
# Options :
#           -a make archive file of the presentation (source, slides, images)
#           -d HTML slides destination directory
#           -h height in percentage of the central table / page
#           -i images directory
#           -t table of contents
#           -V give the version number and exit
# Author : Francois-Xavier Le Bail - fx [dot] LeBail {chez} free [dot] fr
# With vi, you should use : set ts=4 sw=4
#------------------------------------------------------------------------------

# SCREEN MESSAGES
# You can add messages in your language here and update the "language" variable

language=en 

case "$language" in
fr)
# FRENCH
	USAGE="Usage: `basename $0` [-a] [-d rep_destination] [-h hauteur_table] [-i rep_images] [-t] [fichier_source]"
	done_msg="fait"
	empty_file_msg="est vide"
	file_not_readable_msg="n'est pas lisible"
	making_archive_file_msg="Fabrication de l'archive"
	number_arg_required_msg="Argument numerique requis"
	test_images_existence_msg="Verification de l'existence des images"
	illegal_option_msg="Option illegale"
	error_tag_msg="Erreur balise pour"   # trois mots pour printf
	page_status_msg_1="page"
	page_status_msg_2="lignes"
	;;
en)
# ENGLISH
	USAGE="Usage: `basename $0` [-a] [-d destination_dir] [-h table_height] [-i images_dir] [-t] [source_file]"
	done_msg="done"
	empty_file_msg="is empty"
	file_not_readable_msg="is not legible"
	making_archive_file_msg="Making archive file"
	number_arg_required_msg="Numeric argument required"
	test_images_existence_msg="Testing images existence"
	illegal_option_msg="illegal option"
	error_tag_msg="Error tag for"       # three words for printf
	page_status_msg_1="page"
	page_status_msg_2="lines"
	;;
*)
	echo "Unknown language" >&2
	exit 7
esac

#------------------------------------------------------------------------------
# WARNING with EXPR, some words are keywords : match, substr, index, length
# we NEED "x$word" : 'x...'
# we need also this to avoid "expr: syntax error" when word is "(" or ")"

#------------------------------------------------------------------------------
name_file() {
#
# write the destination directory and the file name formated
# example : destination/slide009.html
#
	printf "$destination_dir/$prefix%03d.$suffix" $1
}

#------------------------------------------------------------------------------
name_link() {
#
# write a link name
# example : slide009.html
#
	printf "$prefix%03d.$suffix" $1
}

#------------------------------------------------------------------------------
redirect() {
#
# redirect the standard output for the program, to write a slide
#
	exec > `name_file $page_number`
}

#------------------------------------------------------------------------------
end_dl_tag() {
#
# write the </DL> tags
#
	i=$current_level
	while [ "$i" -gt "$next_level" ]; do
		echo "</DL>"
		i=`expr $i - 1`
	done
}

#------------------------------------------------------------------------------
escape_html() {
#
# escape the &, <, > characters for HTML
#
		echo "$1" |
		sed 's:&:\&amp;:g' |
		sed -e 's:<:\&lt;:g' -e 's:>:\&gt;:g'
}

#------------------------------------------------------------------------------
test_1_image_existence() {
#
# test the existence of one image
#
	cd "$destination_dir"
	if [ ! -r "$images_dir/$1" ]; then
		echo >&2
		echo "\"$images_dir/$1\" $file_not_readable_msg" >&2
		exit 5
	fi
	cd "$actual_dir"
}

#------------------------------------------------------------------------------
#
# test the existence of images
#
test_images_existence() {
	echo >&2
	echo -n "$test_images_existence_msg " >&2
	for i in start.gif prev.gif next.gif blue.gif green.gif lblue.gif "$logo_img"; do
		echo -n "." >&2
		test_1_image_existence $i
	done
	echo " $done_msg." >&2
}

#------------------------------------------------------------------------------
page_status () {
#
# print a page status (page number, nb lines in the page, HTML title)
#
	printf "%s %2d  (%2d %s)  %s\n" \
		$page_status_msg_1 $page_number $page_line_number $page_status_msg_2 \
		"$html_title" >&2
}

#------------------------------------------------------------------------------
write_html_header() {
#
#  write the html header of the page.
#
	cat <<-HeRe
		<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN">
		<!-- Format with table_height = $table_height% -->
		<HTML>
		<HEAD>
		<META name="generator" content="slides.sh / fx [dot] LeBail {chez} free [dot] fr (GNU GPL)">
		<META name="author" content="$author">
		<META name="keywords" content="$keywords">
		<META name="description" content="$description">
		<META http-equiv="Content-Type" content="text/html; charset=$charset">
	HeRe
	if [ "$auto_pull_wait" ]; then
		if [ "$auto_pull_wait" -gt 0 ]; then
			cat <<-HeRe
				<META http-equiv="refresh" content="$auto_pull_wait; URL=$next_link">
			HeRe
		fi
	fi
	cat <<-HeRe
		<LINK rev="made" href="mailto:$email">
		<TITLE>$html_title</TITLE>
		</HEAD>
	HeRe
}

#------------------------------------------------------------------------------
write_navigation_line() {
#
# write the navigation line
#
# call : name_link
#
	if [ $page_number -eq 1 ]; then
		prev_link="`name_link $total_page_number`"
	else
		prev_link="`name_link $previous_page_number`"
	fi

	if [ $page_number -eq "$total_page_number" ]; then
		next_link="`name_link 1`"
	else
		next_link="`name_link \`expr $page_number + 1\``"
	fi

	cat <<-HeRe
		<!-- - - - - - - - - - - Navigation line - - - - - - - - - - -->
		<TABLE width="100%" bgcolor="$color" cellspacing=0 cellpadding=0>
			<TR>
				<TD>
				<TABLE width="100%" cellspacing=0 cellpadding=0>
					<TR>
	HeRe
	if [ $page_number -ne 1 -a $start_img_presence = yes ]; then
		cat <<-HeRe
						<TD>
							<A href="`name_link 1`">
							<IMG src="$images_dir/start.gif" alt="[$start_img_alt]" border=0 height=28 width=40>
							</A>
						</TD>
		HeRe
	fi
	cat <<-HeRe
						<TD align=center width="100%">
							<FONT color="$page_title_color" face="$font_face" size="+4">
							$page_title
							</FONT>
						</TD>
						<TD>
							<A href="$prev_link">
							<IMG src="$images_dir/prev.gif" alt="[$previous_img_alt]" border=0 height=28 width=40>
							</A>
						</TD>
						<TD>
							<A href="$next_link">
							<IMG src="$images_dir/next.gif" alt="[$next_img_alt]" border=0 height=28 width=40>
							</A>
						</TD>
				</TABLE>
		</TABLE>
	HeRe
}

#------------------------------------------------------------------------------
process_header() {
#
# write the header of the page and the beginning of the body (navigation line)
#
# call : write_html_header, write_navigation_line
#
	body_tag='<BODY bgcolor="'"$background_color"'"'

	if [ "$background_img" ]; then
		test_1_image_existence "$background_img"
		body_tag="$body_tag"' background="'"$images_dir/$background_img"'">'
	else
		body_tag="$body_tag"'>'
	fi

	write_html_header

	echo "$body_tag"

	write_navigation_line
}

#------------------------------------------------------------------------------
process_footer() {
#
# write the footer of the page. organization informations and page number 
#
# call : page_status
#
	cat <<-HeRe
				</TD>
			</TR>
		</TABLE>
		<!-- - - - - - - - - - - Footer - - - - - - - - - - -->
		<HR align=left noshade size="1" width="$ruler_width">
		<TABLE border=0 width="100%" cellspacing=0 cellpadding=0>
			<TR>
				<TD align=left>
					<ADDRESS>
					<FONT size="-1">
					<IMG src="$images_dir/$logo_img" alt="$logo_alt" align=left height=$logo_height width=$logo_width>
	HeRe
	echo -n "$organization"
	[ "$adr1" ] && echo -n " - $adr1"
	[ "$adr2" ] && echo -n " - $adr2"
	echo "<BR>"
	echo -n "$email_text : "
	if [ $page_number -eq 1 -o $page_number -eq $total_page_number ]; then
		cat <<-HeRe
			<A href="mailto:$email">$email</A>
		HeRe
	else
		echo -n "$email"
	fi
	[ "$tel" ] && echo -n " - $tel_text : $tel"
	[ "$fax" ] && echo -n " - $fax_text : $fax"
	echo
	cat <<-HeRe
					</FONT>
					</ADDRESS>
				</TD>
				<TD align=right>
					<FONT face="$font_face" size="-1"><EM>- $page_number -</EM></FONT>
		</TABLE>
		</BODY>
		</HTML>
	HeRe
	page_status
	if [ $page_number -ne "$total_page_number" ]; then
		previous_page_number=$page_number
	fi
}

#------------------------------------------------------------------------------
process_page_level() {
#
# end the preceding page, begin the current page
#
# call : process_header, process_footer, redirect, write_words
#
	if [ $page_number != 0 ]; then
		process_footer
	fi

	page_number=`expr $page_number + 1`
	page_line_number=0

	if [ $page_number -ge 2 -a $page_number -le $max_num_page_contents ]; then
		in_table_of_contents=yes
		table_width=$table_width_toc
	else
		in_table_of_contents=no
		table_width=$table_width_std
	fi
	[ "$DEBUG" = 4 ] && echo "	table width : $table_width" >&2

	redirect

	html_title="$current_line"
	if [ $page_number = 1 ]; then
		description="$html_title"
		presentation_title="$html_title"
		page_title='&nbsp;'
	else
		description="$presentation_title - $html_title"
		page_title="$html_title"
	fi

	process_header

	cat <<-HeRe
		<!-- - - - - - - - - - - Central table - - - - - - - - - - -->
		<!-- The "height" attribute of TABLE tag is not standard, but OK with Netscape and Konqueror ... -->
		<TABLE align=center border=0 width="$table_width%" height="$table_height%" cellspacing=0 cellpadding=5>
			<TR>
				<TD valign=middle>
	HeRe
	if [ $page_number = 1 ]; then
		# the <BR> tags are useful for some non Netscape browser (kfm, etc.)
		cat <<-HeRe
					<BR><BR><BR>
					<P align=center>
					<FONT color="$presentation_title_color" face="$font_face" size="+4">
					<STRONG>
					$presentation_title
					</STRONG>
					</FONT>
					<P align=center>
					<FONT color="$presentation_author_color" face="$font_face" size="+3">
					$author
					</FONT>
					<BR><BR><BR><BR>
		HeRe
	else
		if [ "$complementary" ]; then
			page_line_number=`expr $page_line_number + 1`
			cat <<-HeRe
					<P>
					<FONT color="$complementary_text_color" face="$font_face" size="+3">
			HeRe
			write_words $complementary
			cat <<-HeRe
					</FONT>
			HeRe
		fi
	fi
}

#------------------------------------------------------------------------------
write_line() {
#
# write a line with the bullet image
#
# call : write_words
#
	image=$1
	img_size=$2
	char_size=$3
	alt=$4
	page_line_number=`expr $page_line_number + 1`
	if [ "$current_level" -gt "$previous_level" ]; then
		echo "<DL>"
	fi
	echo "<DT><DD>"

	tag_p=`expr "$current_line" : '.*\(!p\)'`
	tag_i=`expr "$current_line" : '.*\(![i]\)'`
	tag_n=`expr "$current_line" : '.*\(!n\)'`
	tag_s=`expr "$current_line" : '.*\(![s#$]\)'`
	tag_S=`expr "$current_line" : '.*\(!S\)'`

	if [ "$tag_p" ]; then
		echo -n "<BR>"
	fi

	if [ "$tag_S" -a ! "$tag_n" ]; then
		if [ "$in_source" = yes ]; then
			in_source=
		else
			in_source=yes
		fi
	fi

# Table of Contents links
	if [ $in_table_of_contents = yes ]; then
		link_page_number=`expr $max_num_page_contents + $page_line_number + \
			\( $page_number - 2 \) * $max_line_contents`
		[ "$DEBUG" = 5 ] && echo "> $link_page_number" >&2
		link="`name_link $link_page_number`"
		link_begin="<A href=\"$link\">"
		link_end="</A>"
		border="border=0 "
	else
		link_begin=
		link_end=
		border=
	fi

# if neutral or not image tag and not source tag => color ball
	if [ "$tag_n" -o \( ! "$tag_i"  -a ! "$tag_s" -a ! "$tag_S" \
	   -a ! "$in_source" \) ]; then
		cat <<-HeRe
			$link_begin<IMG src="$images_dir/$image" alt="$alt" ${border}height=$img_size width=$img_size>$link_end
		HeRe
	fi

	cat <<-HeRe
		<FONT face="$font_face" size="$char_size"> 
	HeRe

	write_words $current_line

	echo "</FONT>"
	#echo "</DD>"

	end_dl_tag
}

#------------------------------------------------------------------------------
write_words() {
#
# analyse a tag and write a word
#
	for word in "$@"; do
		[ "$DEBUG" = 3 ] && echo "	>$word<" >&2
		tag=`expr "x$word" : 'x!\(.\)'`
		if [ "$tag" ]; then
			word=`expr "$word" : '!.\(.*\)'`
			[ "$DEBUG" = 2 ] && echo "	$tag	\"$word\"" >&2
			case "$tag" in
	# Bold (strong) characters
			b)
				cat <<-HeRe
					<STRONG>$word</STRONG>
				HeRe
				;;
	# FTP link
			f)
				word_remainder=`echo $word | sed 's;ftp://;;'`
				cat <<-HeRe
					<A href="ftp://$word_remainder">$word</A>
				HeRe
				;;
	# HTTP link
			h)
				word_remainder=`echo $word | sed 's;http://;;'`
				cat <<-HeRe
					<A href="http://$word_remainder">$word</A>
				HeRe
				;;
	# Local (relative) URI link (access with "file", "http" and "ftp" schemes)
			l)
				cat <<-HeRe
					<A href="$word">$word</A>
				HeRe
				;;
	# MAILTO link
			m)
				word_remainder=`echo $word | sed 's;mailto:;;'`
				cat <<-HeRe
					<A href="mailto:$word_remainder">$word</A>
				HeRe
				;;
	# image with parameters
			i)
				IFS="," 
				set $word
				test_1_image_existence $1
				IFS="$OLDIFS"
				alt_attribute=
				if [ "$2" ]; then
					alt=`echo $2 | sed 's/_/ /g'`
					alt_attribute='alt="'"$alt"'"'
				fi
				height_attribute=
				if [ "$3" ]; then
					height_attribute='height="'"$3"'"'
				fi
				width_attribute=
				if [ "$4" ]; then
					width_attribute='width="'"$4"'"'
				fi
				if [ "$5" = "center" ]; then
					echo "<CENTER>"
				fi
				cat <<-HeRe
					<IMG src="$images_dir/$1" align="middle" $alt_attribute $height_attribute $width_attribute>
				HeRe
				if [ "$5" = "center" ]; then
					echo "</CENTER>"
				fi
				;;
	# to make tag neuter
			n)
				echo -n "$word "
				;;
	# Paragraph
			p)
				echo -n "$word "
				;;
	# Source text
			[sS])
				echo -n "$word "
				;;
	# Command text Unix
			\#)
				echo -n "# $word "
				;;
			\$)
				echo -n "$ $word "
				;;
	# Error
			*)
				echo -n "$word "
				printf "\n%s page $page_number (%s \"!$tag\" %s \"$word\")\n" $error_tag_msg >&2
				exit 6
			esac
		else
			echo -n "$word "
		fi
	done
}

#------------------------------------------------------------------------------
#
# analyse the next level and process a source line
#
# call : write_line
#
process_line() {
	# it is 'x<TAB> *' and not 'x<SPACE> *'
	# x always match => level page (0 tab) = 1, 1 tab = 2, 2 tab = 3, etc.
	next_level=`expr "x$next_line" : 'x	*'`
	current_line="`escape_html "$current_line"`"
	IFS="$separator" 
	set $current_line
	current_line="$1"
	complementary="$2"
	IFS="$OLDIFS"
	case "$current_level" in
	1)	process_page_level;;
	2)	write_line blue.gif  14 +3 \*;; #image image_size char_size alt_text
	3)	write_line green.gif 14 +2  -;;
	4)	write_line lblue.gif  5 +2  .;;
	esac
}

#------------------------------------------------------------------------------
make_table_of_contents() {
#
# make the table of contents 
#
	new_source_file=/tmp/slides.tmp.$$
	{
		sed '/^--$/,$d' "$source_file"
		echo "--"
		# title page
		sed -e '1,/^--$/d' -e q "$source_file"
		nb_titles=0
		sed '1,/^--$/d' "$source_file" |
		grep "^[^	]" |                 # only the page level
		grep -v "^$comment_start" |      # but not the comment lines
		{
			read line                    # not the title page
			nb_page_contents=0
			while read line; do
				expr $nb_titles % $max_line_contents > /dev/null || {
					nb_page_contents=`expr $nb_page_contents + 1`
					if [ $nb_page_contents = 1 ]; then
						echo "$table_of_contents_text"
					else
						echo "$table_of_contents_text $table_of_contents_cont"
					fi
				}
				echo "	$line"
				nb_titles=`expr $nb_titles + 1`
			done
			# need save in a file (because of sub-shell)
			echo $nb_page_contents > /tmp/slides.2.tmp.$$
		}
		sed '1,/^--$/d' "$source_file" | tail +2
	} > $new_source_file
	#less $new_source_file
}

#------------------------------------------------------------------------------
#----- MAIN PROGRAM -----------------------------------------------------------
#------------------------------------------------------------------------------

# default values

# miscellaneous
destination_dir=.
images_dir=.
prefix=slide                 # of the HTML file
suffix=html                  # or "htm"
index_name=index             # or "homepage"
table_height=80              # height (in percentage) of central table / page
table_width_std=95           # width (in percentage) central table / page for
                             # standard page
table_width_toc=60           # width (in percentage) central table / page for 
                             # table of contents
font_face="Arial,Helvetica"  # document characters
table_of_contents=no
max_line_contents=12         # max lines by page for table of contents
table_of_contents_text="Table of contents"
table_of_contents_cont="(continued)"
background_color="#ffffff"
presentation_title_color="#0000ff"
presentation_author_color="#000000"
complementary_text_color="#0000ff"

# header HTML
charset="iso-8859-1"

# navigation line
color="#ffffcc"
page_title_color="#000000"
start_img_presence=no        # enable/disable "Start" Button (link to first slide)
start_img_alt="Start"
previous_img_alt="Previous"
next_img_alt="Next"

# HTML footer
tel_text="Telephone"
fax_text="Fax"
email_text="e-mail"

# for comment
comment_start="::"

# for complementary text
separator=""

#save the standard output before redirect it (in function redirect)
exec 3>&1

# if the first argument of "getopts" begin by ":", no english error message
# you can do your own messages in your language
while getopts :ad:h:i:tV option; do
	case $option in
	a)  PACKAGE=yes;;
	d)  cmd_line_destination_dir="$OPTARG";;
	h)  cmd_line_table_height=`expr $OPTARG + 0 2>/dev/null`
		if [ $? = 2 ]; then
			echo "`basename $0`: $number_arg_required_msg" >&2
			exit 3
		fi;;
	i)  cmd_line_images_dir="$OPTARG";;
	t)  cmd_line_table_of_contents=yes;;
	V)  echo "`basename $0` $Version (GNU GPL)"
	    exit 0;;
	\?) echo "$illegal_option_msg : $OPTARG"
		echo "$USAGE" >&2
		exit 4;;
	esac
done
shift `expr $OPTIND - 1`

source_file=${1:-source.txt}
new_source_file="$source_file"

if [ ! -r "$source_file" ]; then
	echo "\"$source_file\" $file_not_readable_msg" >&2
	exit 1
fi

actual_dir=`pwd`

{
	# read the first parameter line 
	read next_line || { echo "\"$source_file\" $empty_file_msg" >&2; exit 2;}
	source_header_line_number=1

	# read all remainder parameter line 
	while [ "$next_line" != "--" ]; do
		source_header_line_number=`expr $source_header_line_number + 1`
	# set variable
		eval $next_line
		read next_line
	done
} < "$source_file"

# the command line option have priority
[ "$cmd_line_table_height" ] && table_height="$cmd_line_table_height"
[ "$cmd_line_destination_dir" ] && destination_dir="$cmd_line_destination_dir"
[ "$cmd_line_images_dir" ] && images_dir="$cmd_line_images_dir"
[ "$cmd_line_table_of_contents" ] && table_of_contents="$cmd_line_table_of_contents"
if [ ! -d "$destination_dir" ]; then
	mkdir -p "$destination_dir"
fi

if [ "$table_of_contents" = yes ]; then
	make_table_of_contents
	nb_page_contents=`cat /tmp/slides.2.tmp.$$`
	rm /tmp/slides.2.tmp.$$
else
	nb_page_contents=0
fi
max_num_page_contents=`expr $nb_page_contents + 1`

total_page_number=`grep "^[^	]" "$new_source_file" | grep -c -v "^$comment_start"`
total_page_number=`expr "$total_page_number" - $source_header_line_number`

if [ "$DEBUG" = 1 ]; then
	echo "Language                       : $language" >&2
	echo "Source file                    : $source_file" >&2
	echo "Destination directory          : $destination_dir" >&2
	echo "Images directory / destination : $images_dir" >&2
	echo "Prefix                         : $prefix" >&2
	echo "Suffix                         : $suffix" >&2
	echo "Index name                     : $index_name" >&2
	echo "Height                         : $table_height %" >&2
	echo "Total page number              : $total_page_number" >&2
	echo "Table of Contents              : $table_of_contents" >&2
	echo "Start image presence           : $start_img_presence" >&2

fi

test_images_existence 
echo >&2

# if email masquerage for '.' and '@'
if [ "$email_masq" ]; then
	set $email_masq
	email=`echo "$email" | sed -e "s:\.:$1:g" -e "s:@:$2:g"`
fi

# IFS= is necessary to not lose the tabs a the beginning of lines when read
# the presence of tabs => level
OLDIFS="$IFS"
IFS=

sed '1,/^--$/d' "$new_source_file" |
grep -v "^	*$comment_start" |
{
	# read the first presentation line 
	read next_line || { echo "\"$new_source_file\" $empty_file_msg" >&2; exit 2;}
	current_line="$next_line"
	# it is 'x<TAB> *' and not 'x<SPACE> *'
	# x always match => level page (0 tab) = 1, 1 tab = 2, 2 tab = 3, etc.
	current_level=`expr "x$current_line" : 'x	*'`
	previous_level=

	page_number=0
	in_source=

	# read all remainder presentation line 
	while read next_line; do
		IFS="$OLDIFS"
		process_line
		current_line="$next_line"
		previous_level=$current_level
		current_level=$next_level
		IFS=
	done 
	next_line=
	process_line
	process_footer
}

#restore the standard output in case (tar -v for example)
exec 1>&3

ln -f `name_file 1` "$destination_dir/$index_name.$suffix"

[ "$table_of_contents" = yes ] && rm "$new_source_file"

#------------------------------------------------------------------------------
# make a tar.gz package of source, slides and images

if [ "$PACKAGE" = yes ]; then
	echo >&2
	echo -n "$making_archive_file_msg \"/tmp/$prefix$$.tgz\" ..." >&2
	# enable the "Pathname Expansion"
	set +f
	tar zcf /tmp/$prefix$$.tgz \
		"$source_file" \
		$destination_dir/$prefix[0-9][0-9][0-9].$suffix \
		$destination_dir/$index_name.$suffix \
		$destination_dir/$images_dir/*
	echo " $done_msg." >&2
fi
